;+
;  PRO themis_config
;
;  This procedure serves as the themis configuration file.  It sets global (system)
;  variables and initializes devices
;
;  This procedure will define the location of data files and the data server.
;  This procedure is intended to be called from within the "THM_INIT" procedure.
;
;  This should be the only THEMIS file that requires modification for use in different
;  locations.
;
;  There is no need to modify this file if:
;     - Your computer is an SSL UNIX machine that mounts "/disks/data/"   (i.e. ALL Linux and Solaris machines at SSL)
;     - You use a portable computer that will be caching files on a local hard drive.
;
;
;  Settings  in this file will be overridden by settings in the environment.
;  (see setup_themis or setup_themis_bash for examples of setting environment
;  variables on UNIX-like systems.  The environment can also be set on Windows
;  systems.)
;
;  KEYWORDS
;     no_color_setup   added to prevent cronjob to crash, hfrey, 2007-02-10
;
;-
pro thm_config,caching=caching,colortable=colortable,$
           no_color_setup=no_color_setup


;                        THEMIS SPECIFIC INITIALIZATION
;====================================================================================


; Location of the data Server:
!themis.remote_data_dir = 'http://sprg.ssl.berkeley.edu/data/themis/'

;;;;;;;;;;;
; SECTION 1
;;;;;;;;;;;


; The remainder of this file will determine  defaults based on machine type
; It will not be executed if one of the lines in SECTION 1 has been
; commented out.

;--->  Please edit the following line.
;--->    caching=1 to use local cache.  Preferable for LAPTOP or Non-SSL user.
;--->    caching=0 to use only master copy
;caching = 1

if n_elements(caching) eq 0 then $
   caching = 1 - file_test('/disks/data/themis/.themis_master',/regular)  ; this file should only exist on the master directory at SSL


   if !version.os_family  eq 'Windows' then begin
      if CACHing then begin
         ;;-->   Location of local cache for WINDOWS
         ;;      Edit the following line to point to a writeable directory.
         if file_test('E:\data\themis\',/direc,/write) then !themis.local_data_dir = 'E:\data\themis\'  else  $
         !themis.local_data_dir = 'C:\data\themis\'
         !themis.no_download = 0
      endif else begin
         ;;-->   Mount point of master data directory for WINDOWS
         ;;      E.g. At SSL, justice runs a read-only Samba server which
         ;;      provides direct access to //sprg.ssl.berkeley.edu/data/themis
         !themis.local_data_dir = '\\justice\data\themis\'
         !themis.no_download = 1
      endelse
   endif

   if !version.os_family eq 'unix' then begin

      if caching then begin
         ;;-->  Location of local cache on your laptop for UNIX (or Mac)
         ;;     OR for desktops remote from SSL, this could be edited to
         ;;     point to a shared data area where many users can download
         ;;     data from the master data server.
         !themis.local_data_dir = '~/data/themis/'
         !themis.no_download = 0
      endif else begin
         ;;-->  Location of NFS mount point of master data directory.
         !themis.local_data_dir = '/disks/data/themis/'
         !themis.no_download = 1
      endelse

   endif




; Settings of environment variables can override thm_config
if getenv('THEMIS_DATA_DIR') ne '' then $
   !themis.local_data_dir = getenv('THEMIS_DATA_DIR')

if getenv('THEMIS_REMOTE_DATA_DIR') ne '' then $
   !themis.remote_data_dir = getenv('THEMIS_REMOTE_DATA_DIR')

; Do not do color setup if taken care for already
if not keyword_set(no_color_setup) then begin

;                        GLOBAL SYSTEM VARIABLES and CONFIGURATIONS  (NON-THEMIS)
;====================================================================================
; Set global system variables:
; Please note: These settings will affect all IDL routines, NOT JUST THEMIS routines!

  if n_elements(colortable) eq 0 then colortable = 34     ; default color table

;                        Define POSTSCRIPT color table
  old_dev = !d.name             ;  save current device name
  set_plot,'PS'                 ;  change to PS so we can edit the font mapping
  loadct2,colortable
  device,/symbol,font_index=19  ;set font !19 to Symbol
  set_plot,old_dev              ;  revert to old device

;                        Color table for ordinary windows
  loadct2,colortable

;              Make black on white background
  !p.background = !d.table_size-1                   ; White background   (color table 34)
  !p.color=0                                        ; Black Pen
  !p.font = -1                                      ; Use default fonts


  if !d.name eq 'WIN' then begin
    device,decompose = 0
  endif

  if !d.name eq 'X' then begin
    ; device,pseudo_color=8  ;fixes color table problem for machines with 24-bit color
    device,decompose = 0
    if !version.os_name eq 'linux' then device,retain=2  ; Linux does not provide backing store by default
  endif

endif	; no_color_setup

;===========  debugging options

if !prompt eq 'IDL> ' then !prompt = 'THEMIS> '

;  !quiet=1            ; if !quiet ==1 then  error messages are suppressed


;  dummy = ptrace(option=0)    ; uncomment to provide minimal debugging info
;  dummy = ptrace(option=3)    ; uncomment to provide maximum debugging info


;============= Useful TPLOT options

tplot_options,window=0            ; Forces tplot to use only window 0 for all time plots
;tplot_options,'verbose',1         ; Displays some extra messages
;tplot_options,'psym_lim',100      ; Displays symbols if less than 100 point in panel
;tplot_options,'ygap',.5


;!warn.obs_routines = 1
;!warn.OBS_SYSVARS = 1
;!warn.PARENS = 1
;!warn.TRUNCATED_FILENAME = 1



end
