;+
;Procedure: THM_CAL_FBK
;
;Purpose:  Converts raw FBK (Filter Bank)  data into physical quantities.
;keywords:
;   /VERBOSE or VERBOSE=n ; set to enable diagnostic message output.
;		higher values of n produce more and lower-level diagnostic messages.
;   /ALL
;
;Example:
;   thm_cal_fbk, /all
;
;Notes:
;	-- Changes between signal sources are handled;
;		source info from HED data should be used to get actual units of a given spectrum.
;	-- fixed, nominal calibration pars used (gains and frequency responses), rather than proper time-dependent parameters.
;
; $LastChangedBy: jimm $
; $LastChangedDate: 2007-02-13 14:29:46 -0800 (Tue, 13 Feb 2007) $
; $LastChangedRevision: 408 $
; $URL $
;-
pro thm_cal_fbk, scs=scs, names=names, all=all, files=files,trange=trange, $
	discard_raw=discard_raw, valid_names=valid_names, verbose=verbose

thm_init

fbk_valid_names = [ 'fbh', 'fb1', 'fb2' ]

if arg_present( valid_names) then begin
    valid_names = fbk_valid_names
	message, /info, string( strjoin( fbk_valid_names, ','), format='( "Valid names:",X,A,".")')
	return
endif

if not keyword_set(scs) then scs = ['a','b','c','d','e']

if not keyword_set( names) then begin
	names = fbk_valid_names
endif

for s=0L,n_elements(scs)-1L do begin
     sc = scs[s]
     for n=0L, n_elements( names)-1L do begin
        name = names[ n]
		tplot_var = thm_tplot_var( sc, name)
		get_data, tplot_var, data=d, limit=l, dlim=dl

		; default behavior is to preserve the RAW data under a different TPLOT variable name.
		if not keyword_set( discard_raw) then begin
			str_element, dl, 'data_type', 'raw', /add
			tplot_var_raw = string( tplot_var, 'raw', format='(A,"_",A)')
			store_data, tplot_var_raw, data=d, limit=l, dlimit=dl
		endif


		; handle regular FB and HF channels differently.
		switch strlowcase( name) of
			'fb1':
			'fb2':	begin
				tplot_var_src = tplot_var + '_src'
				get_data, tplot_var_src, data=d_src, limit=l_src, dlim=dl_src

				; check that returned data and hed structures are structures (get_data returns 0 if no TPLOT variable exists).
				if (size( d, /type) eq 8) and (size( d_src, /type) eq 8) then begin

					; Determine what signal sources were selected over the given time interval,
					; and apply the appropriate RAW->PHYS conversion factors.
					fb_sel = d_src.y
					find_const_intervals, fb_sel, nint=nint, ibeg=ibeg, iend=iend
					res = fltarr( size( d.y, /dimensions)) + !values.f_nan
					for ii=0L,nint-1L do begin
						t_hdr = [ d_src.x, !values.d_infinity]
						tbeg = t_hdr[ ibeg[ ii]]
						tend = t_hdr[ iend[ ii] + 1L]
						; find all the spectra from the beginning of the first packet of the interval,
						; to the end of the last packet of the interval
						; (actually just before the beginning of the first packet of the next interval).
						idx = where( d.x ge tbeg and d.x lt tend, icnt)
						if icnt gt 0 then begin
							thm_get_fbk_cal_pars, tbeg, tend, fb_sel[ ibeg[ ii]], cal_pars=cp
							res[ idx, *] = cp.gain*((1.0 + fltarr( icnt))#cp.freq_resp[ *])*(float(d.y[ idx, *]))
						endif
					endfor	; loop over constant source intervals.

					; update the DLIMIT elements to reflect RAW->PHYS transformation, coordinate system, etc.
					str_element, dl, 'data_type', 'calibrated', /add
					str_element, dl, 'coord_sys', 'sensor', /add

					units = '<|nT|>,<|V|>, or <|mV/m|>'
					str_element, dl, 'ytitle', string( tplot_var, units, format='(A,"!C!C[",A,"]")'), /add
					str_element, dl, 'units', units, /add
					str_element, dl, 'labels', [ '3 Hz', '12 Hz', '48 Hz', '192 Hz', '768 Hz', '3072 Hz'], /add
					str_element, dl, 'labflag', 1, /add
					str_element, dl, 'colors', [ 1, 2, 3, 4, 5, 6]
					str_element, dl, 'cal_par_time', cp.cal_par_time, /add

					; store the transformed spectra back into the original TPLOT variable.
					store_data, tplot_var, $
						data = { x:d.x, y:res, v:d.v }, $
						lim=l, dlim=dl

				endif else begin	; necessary TPLOT variables not present.
					if verbose then $
						message, /info, $
							string( tplot_var, tplot_var_src, $
							format='("necessary TPLOT variables (",A,X,A,") not present for RAW->PHYS transformation.")')
				endelse
				break
			end	; end of { FB1, FB2} calibration clause.
		'fbh':	begin	; FBH (HF or AKR band filter).
				; check that returned data and hed structures are structures (get_data returns 0 if no TPLOT variable exists).
				if (size( d, /type) eq 8) then begin
					cp = { $
						gain:1.0, freq_resp:3.0, units:'V [100-400 kHz]', $
						offset:222.13, slope: 22.58, $
						cal_par_time:'2002-01-01:00:00:00' $
					}

					; convert digital log-amp output to differential volts at input to V1-V2.
					res = exp((float(d.y)-cp.offset)/cp.slope)

					; convert differential volts at input to V1-V2 to differential volts, sphere1-sphere2.
					res = cp.gain*cp.freq_resp*res

					; update the DLIMIT elements to reflect RAW->PHYS transformation, coordinate system, etc.
					str_element, dl, 'data_type', 'calibrated', /add
					str_element, dl, 'coord_sys', 'sensor', /add

					units = 'V [100-400 kHz]'
					str_element, dl, 'ytitle', string( tplot_var, units, format='(A,"!C!C[",A,"]")'), /add
					str_element, dl, 'units', units, /add
					str_element, dl, 'labels', [ 'avg', 'pk'], /add
					str_element, dl, 'labflag', 1, /add
					str_element, dl, 'colors', [ 1, 2]
					str_element, dl, 'cal_par_time', cp.cal_par_time, /add

					; store the transformed spectra back into the original TPLOT variable.
					store_data, tplot_var, $
						data = { x:d.x, y:res, v:d.v }, $
						lim=l, dlim=dl

				endif else begin	; necessary TPLOT variables not present.
					if verbose then $
						message, /info, $
							string( tplot_var, $
							format='("necessary TPLOT variable (",A,") not present for RAW->PHYS transformation.")')
				endelse
			break
		end
		else:	begin	; improperly defined name of FBK quantity.
		end
	endswitch

	endfor	; loop over names.
endfor	; loop over spacecraft.

end
