;+
;procedure: thm_cal_fgm
;
;Purpose: applies calibration to THEMIS fluxgate magnetometer data
;
;         searches for the current calibration in ASCII file
;         takes calibration that has time stamp before or at the start of the data
;         converts data to nano Tessla
;         applies the calibration
;         corrects for phase shift from averaging before despinning
;INPUTS:
;
;         name_thx_fgx_in   --> input data (t-plot variable name)
;         name_thx_fgx_hed  --> header information for input data (t-plot variable name)
;         name_thx_fgx_out  --> name for output (t-plot variable name)
;         pathfile          --> path and filename of the calibration file
;
;
;keywords:
;   /fgh : calibrates high res data
;   /fgl : calibrates fgl-data
;Example:
;      tha_cal_fgm('tha_fgl','tha_fgl_hed','tha_fgl_ssl','C:\Themis\calfiles\tha_V01.cal',/fgl)
;Notes: under construction!!
;
;Written by Hannes Schwarzl
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-02-02 11:35:42 -0800 (Fri, 02 Feb 2007) $
; $LastChangedRevision: 277 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/spacecraft/fields/thm_cal_fgm.pro $
;-
pro thm_cal_fgm,name_thx_fgx_in,name_thx_fgx_hed,name_thx_fgx_out,pathfile,fgh=fgh,fgl=fgl



; get the data using t-plot names
get_data,name_thx_fgx_in,data=thx_fgx_in
get_data,name_thx_fgx_hed,data=thx_fgx_hed



; make STRUCTRE thx_fgx; converting egineering units from LONG to DOUBLE
thx_fgx = CREATE_STRUCT('X',thx_fgx_in.X , 'Y', DOUBLE(thx_fgx_in.Y), 'V', thx_fgx_in.V)


;some constants
isFgl=0
isFgh=0

; scale factor
kr=2.980232238769531E-3   ;kr=25000.0/2^23 --> see *CALPROC*.doc

; check input

if keyword_set(fgh) then begin
	PRINT,'fgh'
	isFgh=1
endif

if keyword_set(fgl) then begin
	PRINT,'fgl'
	isFgl=1
endif

sumInputs=isFgl+isFgh

IF sumInputs eq 0 THEN begin
	PRINT,'Please set one of the following keywords: /fgh, /fgl'
	stop
ENDIF

IF sumInputs gt 1 THEN begin
	PRINT,'only one of the following keywords: /fgh, /fgl can be set'
	stop
ENDIF


; get array sizes
count=SIZE(thx_fgx.X,/N_ELEMENTS)
countHed=SIZE(thx_fgx_HED.X,/N_ELEMENTS)


PRINT,'get header information ...'

; get the information from the header
fgmhed=thx_fgx_HED.Y[*,12:15] ; store relevant header information

PRINT,'get sampling frequencies ...'
; get fs [vector]
if isFgh eq 1 THEN begin
	 fs = findgen(countHed)
	 fs(*)= 128
endif else begin fs=float(2^(2+(fgmhed[*,3] and '111'B)))
endelse

PRINT,'get filter-modes ...'
; get filtermode [vector]
ff=ishft('F0'X and fgmhed[*,1],-6)

PRINT,'done getting header information'


;read the calibration file

;================
;to be done .....
;================

PRINT,'read calibration file:'
PRINT,pathfile

ncal=file_lines(pathfile)
;define variables
calstr=strarr(ncal)
spinperi=dblarr(ncal)
offi=dblarr(ncal,3)
cali=dblarr(ncal,9)
spinperii=dblarr(1)
offii=dblarr(3)
calii=dblarr(9)
utci='2006-01-01T00:00:00.000Z'
utc=dblarr(ncal)
utcStr=strarr(ncal)

openr,2,pathfile
readf,2,calstr
close,2
for i=0,ncal-1 DO BEGIN
	calstri=calstr(i)
	reads,calstr(i),utci,offii,calii,spinperii,format="(a25,3f5,9f9,f9)"
	offi(i,*)=offii
	cali(i,*)=calii
	spinperi(i)=spinperii
	utcStr(i)=utci
	;translate time information
	STRPUT, utci, '/', 10
	utc(i)=time_double(utci)
ENDFOR



PRINT,'done reading calibration file'


PRINT,'search calibration for selected time interval ...'

calIndex=0
compTime=utc(0)
refTime=thx_fgx.X(0)
i=0
WHILE ((compTime lt reftime) && (i lt ncal-1)) DO BEGIN
	i=i+1
	compTime=utc(i)
	IF (compTime gt reftime) THEN BEGIN
		i=i-1
		BREAK
	ENDIF
ENDWHILE

PRINT, 'Select calibration from:'
PRINT, utcStr(i)

offs=offi(i,*)
calim=TRANSPOSE(cali(i,*))
calm=[[calim[0:2]],[calim[3:5]],[calim[6:8]]]
spinper=spinperi(i)

PRINT,'offsets: ',TRANSPOSE(offs)
PRINT,'cal matrix: '
PRINT,calm
PRINT,'spin period: ',spinper



PRINT,'apply scale factor ...'
;apply scale factor
thx_fgx.Y=thx_fgx.Y*kr

PRINT,'apply calibration ...'

;apply calibration matrix and offset
;lastfs=0

i=0
ffi=ff[i]
fsi=fs[i]
timei=thx_fgx_HED.X[i]


for j=0L,count-1L do begin

	; update header information if necessary (WHILE loop should work for all possible cases)
	while ((thx_fgx.X[j] gt timei) && (i lt countHed-1)) do begin
		i=i+1
		ffi=ff[i]
		fsi=fs[i]
		timei=thx_fgx_HED.X[i]
	endwhile



	thx_fgx.Y[j,*]=MATRIX_MULTIPLY(calm, thx_fgx.Y[j,*], /BTRANSPOSE)-offs


	;correct for filter
	if ffi eq 2 then begin
    	arg=-!dpi/fsi/spinper
    	dfilt=128./fsi*sin(!dpi/128.d0/spinper)/sin(-arg)
    	dfm=identity(3,/double)
    	dfm[1,1]=dfilt
    	dfm[0,0]=dfilt
    	sarg=sin(arg)
    	carg=cos(arg) ;& print,arg,dfilt
    	delaym=identity(3,/double)
    	delaym[0,1]=sarg
    	delaym[1,0]=-sarg
    	delaym[0,0]=carg
    	delaym[1,1]=carg
    	mfilt=dfm#delaym
    	thx_fgx.Y[j,*]=MATRIX_MULTIPLY(mfilt, thx_fgx.Y[j,*],/BTRANSPOSE)
	endif
endfor

PRINT,'calibration applied'
PRINT,'done'


store_data,name_thx_fgx_out,data=thx_fgx

;return,thx_fgx
end
