;+
;Procedure: THM_CAL_FIT
;
;Purpose:  Converts raw FIT parameter data into physical quantities.
;keywords:
;   /VERBOSE or VERBOSE=n ; set to enable diagnostic message output.
;		higher values of n produce more and lower-level diagnostic messages.
;   /ALL
;
;Example:
;   thm_cal_fit, /all
;
;Notes:
;	-- FGM range changes are not handled properly; RANGE=0 is assumed.
;	-- fixed, nominal calibration pars used, rather than proper time-dependent parameters.
;
; $LastChangedBy: jimm $
; $LastChangedDate: 2007-02-13 14:29:46 -0800 (Tue, 13 Feb 2007) $
; $LastChangedRevision: 408 $
; $URL $
;-
pro thm_cal_fit, scs=scs, names=names, all=all, files=files,trange=trange, valid_names=valid_names, verbose=verbose

thm_init

fit_valid_names = [ 'fit']

if arg_present( valid_names) then begin
    valid_names = fit_valid_names
	message, /info, string( strjoin( fit_valid_names, ','), format='( "Valid names:",X,A,".")')
	return
endif

if not keyword_set(scs) then scs = ['a','b','c','d','e']

if not keyword_set( names) then begin
	names = fit_valid_names
endif

for s=0L,n_elements(scs)-1L do begin
     sc = scs[s]
     for n=0L, n_elements( names)-1L do begin
        name = names[ n]
     	; this call to string should be replaced with a call to the function used to generate the TPLOT variable name in the first place.
		tplot_var = thm_tplot_var( sc, name)
		if keyword_set( verbose) then $
			message, /info, string( tplot_var, format='("working on TPLOT variable",X,A)')

		get_data, tplot_var, data=d, limit=l, dlim=dl
		get_data, tplot_var+'_hed', data=d_hed, limit=l_hed, dlim=dl_hed
		; get_data, tplot_var+'_code', data=d_code, limit=l_code, dlim=dl_code
		; stop

		; check that returned data and hed structures are structures (get_data returns 0 if no TPLOT variable exists).
		if (size( d, /type) eq 8) and (size( d_hed, /type) eq 8) then begin
			hed_data = thm_unpack_hed( 'fit', d_hed.y)
			str_element, dl, 'data_type', 'calibrated', /add
			str_element, dl, 'coord_sys', 'fit', /add
			cpar = { $
				e:{ cal_par_time:'2002-01-01/00:00:00', Ascale:15.0, Bscale:15.0, Cscale:15.0, theta:0.0, sigscale:15.0, Zscale:15.0, units:'V'}, $
				b:{ cal_par_time:'2002-01-01/00:00:00', Ascale:25.e3 , Bscale:25.e3, Cscale:25.e3, theta:0.0, sigscale:25.e3, Zscale:25.e3, units:'nT'} $
			}
			; E-field fit (EFI).
			idx = 0L
			dqd = 'efit'
			units = cpar.e.units
			tplot_var_efit = string( tplot_var, dqd, format='(A,"_",A)')
			str_element, dl, 'ytitle', string( tplot_var_efit, units, format='(A,"!C!C[",A,"]")'), /add
			str_element, dl, 'units', units, /add
			str_element, dl, 'labels', [ 'A', 'B', 'C', 'Sig', '<Ez>'], /add
			str_element, dl, 'labflag', 1, /add
			str_element, dl, 'colors', [ 1, 2, 3, 4, 5], /add
			str_element, dl, 'cal_par_time', cpar.e.cal_par_time, /add

			d.y[ *, 0, idx] = cpar.e.Ascale*d.y[ *, 0, idx]
			d.y[ *, 1, idx] = cpar.e.Bscale*d.y[ *, 1, idx]
			d.y[ *, 2, idx] = cpar.e.Cscale*d.y[ *, 2, idx]
			d.y[ *, 3, idx] = cpar.e.sigscale*d.y[ *, 3, idx]
			d.y[ *, 4, idx] = cpar.e.Zscale*d.y[ *, 4, idx]

			; store the spin fit parameters.
			store_data, tplot_var_efit, $
				data = { x:d.x, y:reform( d.y[ *, *, idx])}, $
				lim=l, dlim=dl

			dqd = 'e'
			tplot_var_e = string( tplot_var, dqd, format='(A,"_",A)')
			str_element, dl, 'ytitle', string( tplot_var_e, units, format='(A,"!C!C[",A,"]")'), /add
			str_element, dl, 'labels', [ 'Ex', 'Ey', 'Ez'], /add
			str_element, dl, 'colors', [ 2, 4, 6], /add

			; store the spin fit E-field vector (only the B, C, and <Ez> parameters).
			store_data, tplot_var_e, $
				data = { x:d.x, y:reform( d.y[ *, [ 1, 2, 4], idx])}, $
				lim=l, dlim=dl

			; B-field fit (FGM).
			idx = 1L
			dqd = 'bfit'
			units = cpar.b.units
			tplot_var_bfit = string( tplot_var, dqd, format='(A,"_",A)')
			str_element, dl, 'ytitle', string( tplot_var_bfit, units, format='(A,"!C!C[",A,"]")'), /add
			str_element, dl, 'units', units, /add
			str_element, dl, 'labels', [ 'A', 'B', 'C', 'Sig', '<Ez>'], /add
			str_element, dl, 'labflag', 1, /add
			str_element, dl, 'colors', [ 1, 2, 3, 4, 5], /add
			str_element, dl, 'cal_par_time', cpar.b.cal_par_time, /add

			; punt on handling FGM range changes for the moment.
			b_range = 0.
			b_range_fac = 1./2.^b_range
			str_element, dl, 'b_range', b_range, /add
			str_element, dl, 'b_range_fac', b_range_fac, /add

			d.y[ *, 0, idx] = b_range_fac*cpar.b.Ascale*d.y[ *, 0, idx]
			d.y[ *, 1, idx] = b_range_fac*cpar.b.Bscale*d.y[ *, 1, idx]
			d.y[ *, 2, idx] = b_range_fac*cpar.b.Cscale*d.y[ *, 2, idx]
			d.y[ *, 3, idx] = b_range_fac*cpar.b.sigscale*d.y[ *, 3, idx]
			d.y[ *, 4, idx] = b_range_fac*cpar.b.Zscale*d.y[ *, 4, idx]

			store_data, tplot_var_bfit, $
				data = { x:d.x, y:reform( d.y[ *, *, idx])}, $
				lim=l, dlim=dl

			dqd = 'b'
			tplot_var_b = string( tplot_var, dqd, format='(A,"_",A)')
			str_element, dl, 'ytitle', string( tplot_var_b, units, format='(A,"!C!C[",A,"]")'), /add
			str_element, dl, 'units', units, /add
			str_element, dl, 'labels', [ 'Bx', 'By', 'Bz'], /add
			str_element, dl, 'colors', [ 2, 4, 6], /add
			store_data, tplot_var_b, $
				data = { x:d.x, y:reform( d.y[ *, [ 1, 2, 4], idx])}, $
				lim=l, dlim=dl

	endif
	endfor	; loop over names.
endfor	; loop over spacecraft.

end
